package com.xxfc.platform.order.biz;

import cn.hutool.core.util.ObjectUtil;
import com.github.wxiaoqi.security.common.biz.BaseBiz;
import com.github.wxiaoqi.security.common.util.CollectorsUtil;
import com.xxfc.platform.order.contant.enumerate.StatisticsStatusEnum;
import com.xxfc.platform.order.entity.OrderRentVehicleReceivedStatistics;
import com.xxfc.platform.order.mapper.OrderRentVehicleReceivedStatisticsMapper;
import com.xxfc.platform.order.pojo.account.OrderAccountBo;
import com.xxfc.platform.order.pojo.account.OrderAccountDetail;
import com.xxfc.platform.order.pojo.dto.OrderDTO;
import com.xxfc.platform.order.pojo.dto.OrderReceivedStatisticsFindDTO;
import com.xxfc.platform.order.pojo.order.CompanyAmountBo;
import com.xxfc.platform.vehicle.feign.VehicleFeign;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 租车订单统计
 *
 * @author libin
 * @email 18178966185@163.com
 * @date 2019-11-08 18:03:42
 */
@Service
@Transactional(rollbackFor = Exception.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OrderRentVehicleReceivedStatisticsBiz extends BaseBiz<OrderRentVehicleReceivedStatisticsMapper, OrderRentVehicleReceivedStatistics> {

    private final BaseOrderBiz baseOrderBiz;
    private final OrderAccountBiz orderAccountBiz;
    private final VehicleFeign vehicleFeign;
    private final int PAY_ORDER = 101;

    /**
     * 租车统计查询
     *
     * @param orderReceivedStatisticsFindDTO
     * @return
     */
    public List<OrderRentVehicleReceivedStatistics> selectOrderReceivedStatistics(OrderReceivedStatisticsFindDTO orderReceivedStatisticsFindDTO) {
        return mapper.selectOrderRentVehicleReceivedStatistics(orderReceivedStatisticsFindDTO);
    }

    /**
     * 租车统计处理
     *
     * @param startDate 开始时间
     * @param endDate   结束时间
     */
    public void orderRentVehicleReceivedStatistics(Date startDate, Date endDate) {
        List<OrderRentVehicleReceivedStatistics> orderRentVehicleReceivedStatisticsList = new ArrayList<>();
        List<String> stisticsActiveState = new ArrayList<>();
        //公司的收入与支出 ---》无状态
        List<CompanyAmountBo> companyAmoutList = new ArrayList<>();
        //根据租车订单类型 时间查询
        List<OrderDTO> orderDTOS = baseOrderBiz.selectOrdersByTypeAndTime(Arrays.asList(StatisticsStatusEnum.ORDER_RENT_VEHICLE_TYPE), startDate, endDate);

      //数据处理 状态组合 按支付状态分组 而后按组合状态
        Map<Boolean, Map<String, List<OrderDTO>>> stateGroupMap = orderDTOS.stream().peek(x -> {
            x.setStateGroup(String.format("%d-%d-%d-%d", x.getCompanyId(),
                    x.getOrderOrigin(),
                    x.getPayWay() == null ? StatisticsStatusEnum.NO_PAY_WAY:x.getPayWay()
                    ,x.getHasPay()));
            stisticsActiveState.add(x.getStateGroup());
        })
                .collect(Collectors.partitioningBy(x -> Objects.nonNull(x.getPayWay()), Collectors.groupingBy(OrderDTO::getStateGroup, Collectors.toList())));

        //订单账目信息
        List<OrderAccountBo> orderAccountBoList = orderAccountBiz.selectByTypeAndDate(StatisticsStatusEnum.ORDER_RENT_VEHICLE_TYPE,startDate, endDate);
        //账目数据处理 状态组合
        Map<String, List<OrderAccountBo>> ordersMap = orderAccountBoList.stream().peek(x -> {
            x.setStateGroup(String.format("%d-%d-%d-%d", x.getCompanyId(),
                    x.getOrderOrigin(),
                    x.getPayWay() == null ? StatisticsStatusEnum.NO_PAY_WAY:x.getPayWay(),
                    x.getHasPay()));
        }).collect(Collectors.groupingBy(OrderAccountBo::getStateGroup, Collectors.toList()));

        //已经支付单
        Set<Map.Entry<String, List<OrderAccountBo>>> ordersSet = ordersMap.entrySet();
        for (Map.Entry<String, List<OrderAccountBo>> orderEntry : ordersSet) {
            String orderKey = orderEntry.getKey();
            List<OrderAccountBo> orderAccountBos = orderEntry.getValue();

            BigDecimal totalAmount = BigDecimal.ZERO;
            BigDecimal refundAmount = BigDecimal.ZERO;

            for (OrderAccountBo orderAccountBo : orderAccountBos) {
                OrderAccountDetail accountDetailEntity = orderAccountBo.getAccountDetailEntity();
                if (orderAccountBo.getAccountType() == PAY_ORDER) {
                    totalAmount.add(accountDetailEntity.getOrderAmount()).add(accountDetailEntity.getDepositAmount());
                } else {
                    refundAmount.add(accountDetailEntity.getOrderAmount()).add(accountDetailEntity.getDepositAmount());
                }
            }
            if (stisticsActiveState.contains(orderKey)){
                OrderRentVehicleReceivedStatistics orderRentVehicleReceivedStatistics = StatisticsStatusEnum.wrapStatisticsObject(startDate, orderKey, new OrderRentVehicleReceivedStatistics());
                orderRentVehicleReceivedStatistics.setTotalAmount(totalAmount.subtract(refundAmount));
                orderRentVehicleReceivedStatistics.setTotalQuantity(stateGroupMap.get(Boolean.TRUE).get(orderKey).size());
                orderRentVehicleReceivedStatisticsList.add(orderRentVehicleReceivedStatistics);
            }else {
                Integer compnayId = StatisticsStatusEnum.getCompnayId(orderKey);
                CompanyAmountBo companyAmountBo = CompanyAmountBo.builder().companyId(compnayId).amount(totalAmount.subtract(refundAmount)).build();
                companyAmoutList.add(companyAmountBo);
            }
        }

        //无状态组合统计对象生成
        Map<Integer, BigDecimal> companyAmountMap = companyAmoutList.stream().collect(Collectors.groupingBy(CompanyAmountBo::getCompanyId,
                CollectorsUtil.summingBigDecimal(CompanyAmountBo::getAmount)));
        Set<Map.Entry<Integer, BigDecimal>> companyAmountSet = companyAmountMap.entrySet();
        for (Map.Entry<Integer, BigDecimal> companyAmountEntry : companyAmountSet) {
            String stateGrp = String.format("%d-%d-%d-%d",companyAmountEntry.getKey(),StatisticsStatusEnum.NO_ORDER_ORIGN,StatisticsStatusEnum.NO_PAY_WAY,StatisticsStatusEnum.NO_ORDER_STATE);
            OrderRentVehicleReceivedStatistics orderRentVehicleReceivedStatistics = StatisticsStatusEnum.wrapStatisticsObject(startDate, stateGrp, new OrderRentVehicleReceivedStatistics());
            orderRentVehicleReceivedStatistics.setTotalQuantity(0);
            orderRentVehicleReceivedStatistics.setTotalAmount(companyAmountEntry.getValue());
            orderRentVehicleReceivedStatisticsList.add(orderRentVehicleReceivedStatistics);
        }

        //未支付单
        List<OrderRentVehicleReceivedStatistics> noPayOrderRentVehicleStatisticsList = createNoPayOrderRentVehicleStatisticsList(startDate, stateGroupMap.get(Boolean.FALSE));
        orderRentVehicleReceivedStatisticsList.addAll(noPayOrderRentVehicleStatisticsList);

        //查询分公司ids
        List<Integer> companyIds = vehicleFeign.findCompanyIdsByAreaId(null);
        //创建剩余状态组合的租车统计对象
        List<OrderRentVehicleReceivedStatistics> otherStatisticsStateGroupList = createOtherStatisticsStateGroupList(startDate,stisticsActiveState, companyIds);
        orderRentVehicleReceivedStatisticsList.addAll(otherStatisticsStateGroupList);
        //保存
        insertMemberReceivedStatisticsBatch(orderRentVehicleReceivedStatisticsList);
    }

    /**
     *
     * @param startDate 时间
     * @param noPayOrdersMap 未支付单map
     * @return
     */
    private List<OrderRentVehicleReceivedStatistics> createNoPayOrderRentVehicleStatisticsList(Date startDate,Map<String, List<OrderDTO>> noPayOrdersMap) {
        List<OrderRentVehicleReceivedStatistics> orderRentVehicleReceivedStatisticsList = new ArrayList<>();
        if (noPayOrdersMap==null || noPayOrdersMap.isEmpty()){
            return orderRentVehicleReceivedStatisticsList;
        }
        Set<Map.Entry<String, List<OrderDTO>>> noPayOrderSet = noPayOrdersMap.entrySet();
        for (Map.Entry<String, List<OrderDTO>> noPayOrderEntry : noPayOrderSet) {
            String noPayOrderStateGroup = noPayOrderEntry.getKey();
            List<OrderDTO> noPayOrders = noPayOrderEntry.getValue();
            BigDecimal totalNoPayAmount = noPayOrders.stream().map(OrderDTO::getRealAmount).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            OrderRentVehicleReceivedStatistics orderRentVehicleReceivedStatistics = StatisticsStatusEnum.wrapStatisticsObject(startDate, noPayOrderStateGroup, new OrderRentVehicleReceivedStatistics());
            orderRentVehicleReceivedStatistics.setTotalAmount(totalNoPayAmount);
            orderRentVehicleReceivedStatistics.setTotalQuantity(noPayOrders.size());
            orderRentVehicleReceivedStatisticsList.add(orderRentVehicleReceivedStatistics);
        }
        return orderRentVehicleReceivedStatisticsList;
    }

    /**
     * 创建剩余状态数据
     *
     * @param startDate               时间
     * @param statisticsStateGroups   状态组合 集合
     * @param companyIds              公司ids
     * @return
     */
    private List<OrderRentVehicleReceivedStatistics> createOtherStatisticsStateGroupList(Date startDate,
                                                                                         List<String> statisticsStateGroups,
                                                                                         List<Integer> companyIds) {

        List<OrderRentVehicleReceivedStatistics> orderRentVehicleReceivedStatisticsList = new ArrayList<>();
        //获取剩余状态组合
        List<String> otherStatisticsStateGroup = StatisticsStatusEnum.getOtherStatisticsStateGroup(companyIds, statisticsStateGroups);
        //创建租车统计克隆对象
        OrderRentVehicleReceivedStatistics orderRentVehicleReceivedStatistics = new OrderRentVehicleReceivedStatistics();
        //统计对象的生成
        otherStatisticsStateGroup.stream().map(stateGroup -> {
            OrderRentVehicleReceivedStatistics orderRentVehicleReceivedStatisticsClone = StatisticsStatusEnum.wrapStatisticsObject(startDate, stateGroup, ObjectUtil.cloneByStream(orderRentVehicleReceivedStatistics));
            orderRentVehicleReceivedStatisticsClone.setTotalAmount(BigDecimal.ZERO);
            orderRentVehicleReceivedStatisticsClone.setTotalQuantity(0);
            orderRentVehicleReceivedStatisticsList.add(orderRentVehicleReceivedStatisticsClone);
            return orderRentVehicleReceivedStatisticsClone;
        }).count();

        return orderRentVehicleReceivedStatisticsList;
    }

    /**
     * 批量插入数据  mysql sql语句默认不能超过4M
     *
     * @param orderRentVehicleReceivedStatistics
     */
    public void insertMemberReceivedStatisticsBatch(List<OrderRentVehicleReceivedStatistics> orderRentVehicleReceivedStatistics) {
        orderRentVehicleReceivedStatistics.sort(Comparator.comparing(OrderRentVehicleReceivedStatistics::getCompanyId));
        int orderSize = orderRentVehicleReceivedStatistics.size();
        int sqlAdq = orderSize / StatisticsStatusEnum.DEFAULT_SQL_SIZE;
        int sqlMod = orderSize % StatisticsStatusEnum.DEFAULT_SQL_SIZE;
        sqlAdq = sqlMod == 0 ? sqlAdq : sqlAdq + 1;
        for (int i = 0; i < sqlAdq; i++) {
            int fromIndex = StatisticsStatusEnum.DEFAULT_SQL_SIZE * i;
            int toIndex = StatisticsStatusEnum.DEFAULT_SQL_SIZE * (i + 1);
            toIndex = toIndex > orderSize ? orderSize : toIndex;
            List<OrderRentVehicleReceivedStatistics> orderRentVehicleReceivedStatisticsList = orderRentVehicleReceivedStatistics.subList(fromIndex, toIndex);
            mapper.insertList(orderRentVehicleReceivedStatisticsList);
        }
    }
}